from __future__ import absolute_import
from typing import Dict

from aoa.api.base_api import BaseApi


class TrainedModelApi(BaseApi):

    path = "/api/trainedModels/"

    def find_all(self, projection: str = None, page: int = None, size: int = None, sort: str = None):
        """
        returns all trained models

        Parameters:
           projection (str): projection type
           page (int): page number
           size (int): number of records in a page
           sort (str): column name and sorting order
           e.g. name?asc: sort name in ascending order, name?desc: sort name in descending order

        Returns:
            (dict): all trained model
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list', 'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection', 'page', 'sort', 'size', 'sort']
        query_vals = [projection, page, sort, size, sort]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path,
            header_params,
            query_params)

    def find_by_id(self, trained_model_id: str, projection: str = None):
        """
        returns a trained model

        Parameters:
           trained_model_id (str): trained model id(uuid) to find
           projection (str): projection type

        Returns:
            (dict): trained model
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id, 
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list', 'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + trained_model_id,
            header_params,
            query_params)

    def find_dataset(self, trained_model_id: str, projection: str = None):
        """
        returns dataset of a trained model

        Parameters:
           trained_model_id (str): trained model id(uuid)
           projection (str): projection type

        Returns:
            (dict): dataset
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list', 'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + trained_model_id + '/dataset',
            header_params,
            query_params)

    def find_events(self, trained_model_id: str, projection: str = None):
        """
        returns trained model events

        Parameters:
           trained_model_id (str): trained model id(uuid)
           projection (str): projection type

        Returns:
            (dict): events of trained model
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list', 'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + trained_model_id + '/events',
            header_params,
            query_params)

    def find_by_model_id(self, model_id: str, projection: str = None):
        """
        returns a trained models by model id

        Parameters:
           model_id (str): model id(uuid) to find
           projection (str): projection type

        Returns:
            (dict): trained models
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list', 'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['modelId', 'projection']
        query_vals = [model_id, projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + 'search/findByModelId',
            header_params,
            query_params)

    def find_by_model_id_and_status(self, model_id: str, status: str, projection: str = None):
        """
        returns a trained models by model id

        Parameters:
           model_id (str): model id(uuid) to find
           status (str): model status
           projection (str): projection type

        Returns:
            (dict): trained models
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list', 'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['modelId', 'status', 'projection']
        query_vals = [model_id, status, projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + 'search/findByModelIdAndStatus',
            header_params,
            query_params)

    def train(self, training_request: Dict[str, str]):
        """
        train a model

        Parameters:
           training_request (dict): request to train model

        Returns:
            (dict): job
        """
        header_vars = ['AOA-Project-ID', 'Accept', 'Content-Type']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json']),
            'application/json']
        header_params = self.generate_params(header_vars, header_vals)

        query_params = {}

        self.required_params(['modelId', 'datasetId'], training_request)

        return self.aoa_client.post_request(
            self.path + 'train',
            header_params,
            query_params,
            training_request)

    def evaluate(self, trained_model_id: str, evaluation_request: Dict[str, str]):
        """
        evaluate a model

        Parameters:
           trained_model_id (str): trained model id(uuid) to evaluate
           evaluation_request (dict): request to evaluate trained model

        Returns:
            (dict): job
        """
        header_vars = ['AOA-Project-ID', 'Accept', 'Content-Type']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json']),
            'application/json']
        header_params = self.generate_params(header_vars, header_vals)

        query_params = {}

        self.required_params(['datasetId'], evaluation_request)

        return self.aoa_client.post_request(
            self.path + trained_model_id + '/evaluate',
            header_params,
            query_params,
            evaluation_request)

    def transition(self, trained_model_id: str, transition_request: Dict[str, str]):
        """
        transitions a model from one state to another (evaluated -> approved or rejected -> deployed -> retired etc)

        Parameters:
           trained_model_id (str): trained model id(uuid) to evaluate
           transition_request (dict): request to transition trained model

        Returns:
            (str): status
        """
        header_vars = ['AOA-Project-ID', 'Accept', 'Content-Type']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json']),
            'application/json']
        header_params = self.generate_params(header_vars, header_vals)

        query_params = {}

        self.required_params(['status'], transition_request)

        return self.aoa_client.post_request(
            self.path + trained_model_id + '/transition',
            header_params,
            query_params,
            transition_request)



import json
import uuid
import collections
import os
import shutil
import logging
import yaml

from pathlib import Path
from git import Repo


class ModelUtility(object):

    def __init__(self, base_path: str, repo_template_catalog: str):
        self.base_path = base_path
        self.dir_path, _ = os.path.split(__file__)
        self.template_catalog = os.path.join(self.dir_path, 'model_templates/')
        self.repo_template_catalog = repo_template_catalog
        self.model = collections.OrderedDict()
        self.model_catalog_path = os.path.join(base_path, '')
        self.model_template = "empty"
        self.logger = logging.getLogger(__name__)

    def add_model(self, model_name: str, model_desc: str, lang: str,
                  template: str, base_path: str = None):

        base_path = self.model_catalog_path if base_path is None else os.path.join(base_path, '')
        model_path = os.path.join(base_path + 'model_definitions/', '')

        if not os.path.isdir(model_path):
            raise ValueError("model directory {0} does not exist".format(model_path))

        self.model["id"] = str(uuid.uuid4())
        self.model["name"] = model_name
        self.model["description"] = model_desc
        self.model["language"] = lang

        self.model_template = template

        self.__add_framework_specific_attributes()
        self.__create_model_structure(model_path)

    @staticmethod
    def clone_repository(url, path):
        Repo.clone_from(url, path)

    def init_model_directory(self, path: str = None):
        logging.info('Creating model directory')
        if path is None:
            path = os.path.join(os.path.abspath(os.getcwd()), '')

        self.logger.info('Creating model definitions')

        src = os.path.join(os.path.split(__file__)[0], '') + 'metadata_files'
        src_files = os.listdir(src)
        for file_name in src_files:
            full_file_name = os.path.join(src, file_name)
            if os.path.isfile(full_file_name) and not os.path.exists(os.path.join(path, file_name)):
                shutil.copy(full_file_name, path)

        Path(path + "model_definitions/").mkdir(parents=True, exist_ok=True)
        Path(path + "model_templates/").mkdir(parents=True, exist_ok=True)

        self.logger.info("model directory initialized at {0}".format(path))

    def __add_framework_specific_attributes(self):
        if self.model_template == "pyspark":
            self.model["automation"] = {
                "trainingEngine": "pyspark"
            }

    def __create_model_structure(self, model_path):
        # check if model template exists in templates catalog passed and use that as override if it does. if not, use
        # defaut version in the package template
        model_template_path = os.path.join(self.repo_template_catalog, '') \
                                   + self.model["language"] + "/" + self.model_template

        if os.path.exists(model_template_path):
            template_dir = model_template_path
        else:
            template_dir = self.template_catalog + self.model["language"] + "/" + self.model_template

        model_dir = model_path + self.model["id"]
        shutil.copytree(template_dir, model_dir)

        Path(model_dir + "/notebooks").mkdir(exist_ok=True)

        with open(model_dir + "/model.json", 'w') as f:
            json.dump(self.model, f, indent=4)

    def get_all_template_catalog(self):
        cat, custom_cat = {}, {}

        if self.repo_template_catalog is not None and os.path.exists(self.repo_template_catalog):
            custom_cat = self.__get_template_catalog(self.repo_template_catalog)

        cat = self.__get_template_catalog(self.template_catalog)

        return cat, custom_cat

    def __get_template_catalog(self, model_template_path):
        catalog = {}

        sub_folders = [f for f in os.listdir(model_template_path)
                       if os.path.isdir(os.path.join(model_template_path, f))
                       & (f[0] != '.') & (f.lower() != 'readme.md')]

        for language in sub_folders:
            language_dir = os.path.join(model_template_path, language)
            catalog[language] = []
            for template_type in os.listdir(language_dir):
                if template_type[0] != '.' and template_type.lower() != 'readme.md':
                    catalog[language].append(template_type)
        return catalog

    def read_repo_config(self):
        path = os.path.join(self.base_path, ".aoa/config.yaml")
        if os.path.exists(path):
            with open(path, "r") as handle:
                return yaml.safe_load(handle)

        self.logger.warning("Aoa repo config doesn't exist")
        return None

    def write_repo_config(self, config):
        config_dir = os.path.join(self.base_path, ".aoa")
        Path(config_dir).mkdir(parents=True, exist_ok=True)
        config_file = "{}/config.yaml".format(config_dir)

        with open(config_file, 'w+') as f:
            yaml.dump(config, f, default_flow_style=False)











def train(data_conf, model_conf, **kwargs):
    """Python train method called by AOA framework

    Parameters:
    data_conf (dict): The dataset metadata
    model_conf (dict): The model configuration to use

    Returns:
    None:No return

    """

    hyperparams = model_conf["hyperParameters"]

    # load data & engineer

    print("Starting training...")

    # fit model to training data

    print("Finished training")

    # export model artefacts to models/ folder

    print("Saved trained model")


import pandas as pd
from sklearn.neighbors import KNeighborsClassifier
from sklearn.externals import joblib
import os

def train(data_conf, model_conf, **kwargs):
    """Python train method called by AOA framework

    Parameters:
    data_conf (dict): The dataset metadata
    model_conf (dict): The model configuration to use

    Returns:
    None:No return

    """

    hyperparams = model_conf["hyperParameters"]

    # load data & engineer
    iris_df = pd.read_csv(data_conf['location'])
    features = 'sepallength,sepalwidth,petallength,petalwidth'.split(',')
    X = iris_df.loc[:, features]
    y = iris_df['class']

    print("Starting training...")
    # fit model to training data
    knn = KNeighborsClassifier(n_neighbors=hyperparams['n_neighbors'])
    knn.fit(X,y)
    print("Finished training")

    # export model artefacts to models/ folder
    if not os.path.exists('models'):
        os.makedirs('models')
    joblib.dump(knn, 'models/iris_knn.joblib')
    print("Saved trained model")


import pytest
import docker
import time
import sys
import os
import requests
import logging

logger = logging.getLogger()

myPath = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, myPath + '/../')

client = docker.from_env()
container_name = "test-aoa-img"
max_retries = 10


def pytest_addoption(parser):
    parser.addoption("--img", action="store", help="specify image name")
    parser.addoption("--port", action="store", help="specify port number")


def check_aoa_up(aoa_url, auth):
    retries = 0
    while retries < max_retries:
        try:
            resp = requests.get(
                url=aoa_url + "/admin/health",
                auth=auth
            )
            return resp.status_code
        except:
            logger.info('aoa demo container...waiting')
            retries += 1
            time.sleep(5)
    assert (5 == 4), 'aoa demo container...unable to access'


@pytest.fixture(scope="session")
def setup(request):
    img_name = request.config.getoption("img") if request.config.getoption("img") is not None \
        else "thinkbiganalytics/aoa-demo:2.5.0"
    port_number = request.config.getoption("port") if request.config.getoption("port") is not None \
        else 8080
    from aoa.api_client import AoaClient
    pytest.aoa_client = AoaClient(aoa_url="http://localhost:8080",
                                  auth_user="admin",
                                  auth_pass="admin")
    pytest.aoa_client.set_project_id("23e1df4b-b630-47a1-ab80-7ad5385fcd8d")

    logger.info('aoa demo container...starting')
    client.containers.run(image=img_name,
                          name=container_name,
                          auto_remove=True,
                          privileged=True,
                          ports={
                              '8080/tcp': port_number,
                              '61613/tcp': 61613
                          },
                          volumes={
                               '/var/run/docker.sock': {
                                    'bind': '/var/run/docker.sock'
                            }
                          },
                          detach=True)
    check_aoa_up(pytest.aoa_client.aoa_url, pytest.aoa_client.auth)
    logger.info('aoa demo container...started')

    def fin():
        logger.info('aoa demo container...removing')
        client.containers.list(all=True, filters={
            "name": container_name
        })[0].remove(force=True)
        logger.info('aoa demo container...removed')
    request.addfinalizer(fin)



from __future__ import absolute_import

from aoa.api.base_api import BaseApi


class TrainerApi(BaseApi):

    path = "/api/trainers"

    def find_all(self, projection: str = None):
        """
        returns all trainers

        Parameters:
           projection (str): projection type

        Returns:
            (dict): trainers
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path,
            header_params,
            query_params)



__version__="2.5.3"

# import client
from aoa.api_client import AoaClient

# import APIs into api package
from aoa.api.dataset_api import DatasetApi
from aoa.api.model_api import ModelApi
from aoa.api.project_api import ProjectApi
from aoa.api.trainer_api import TrainerApi
from aoa.api.trained_model_api import TrainedModelApi
from aoa.api.trained_model_event_api import TrainedModelEventApi
from aoa.api.job_api import JobApi
from aoa.api.job_event_api import JobEventApi

# import repo into api package
from aoa.model_utility.model_utility import ModelUtility
from aoa.model_utility.evaluate_model import EvaluateModel
from aoa.model_utility.train_model import TrainModel
from aoa.model_utility.base_model import BaseModel





from __future__ import absolute_import

import json
import os
import yaml
import logging
import requests
from requests.auth import AuthBase
from typing import List, Dict
from base64 import b64encode


class AoaClient(object):
    def __init__(self, **kwargs):
        self.logger = logging.getLogger(__name__)
        self.project_id = None
        self.auth = None, None
        self.aoa_url = None

        self.__parse_aoa_config(**kwargs)

    def __parse_yaml(self, yaml_path: str):
        with open(yaml_path, "r") as handle:
            conf = yaml.safe_load(handle)
        self.__parse_kwargs(**conf)

    def __parse_kwargs(self, **kwargs):
        self.aoa_url = kwargs.get("aoa_url", self.aoa_url)

        if "aoa_credentials" in kwargs:
            self.auth = AoaAuth(kwargs["aoa_credentials"])

        if "auth_user" in kwargs and "auth_pass" in kwargs:
            credentials = b64encode("{}:{}".format(kwargs["auth_user"], kwargs["auth_pass"]).encode()).decode()
            self.auth = AoaAuth(credentials)

        if "verify_connection" in kwargs:
            self.verify_aoa_connection = kwargs["verify_connection"]

    def __parse_env_variables(self):
        self.aoa_url = os.environ.get("AOA_URL", self.aoa_url)

        if "AOA_API_AUTH_USER" in os.environ and "AOA_API_AUTH_PASS" in os.environ:
            credentials = b64encode("{}:{}".format(os.environ["AOA_API_AUTH_USER"], os.environ["AOA_API_AUTH_PASS"]).encode()).decode()
            self.auth = AoaAuth(credentials)

    def set_project_id(self, project_id: str):
        """
        set project id

        Parameters:
           project_id (str): project id(uuid)
        """
        self.project_id = project_id

    def get_current_project(self):
        """
        get project id

        Return:
           project_id (str): project id(uuid)
        """
        return self.project_id

    def __parse_aoa_config(self, **kwargs):
        if "config_file" in kwargs:
            self.__parse_yaml(kwargs['config_file'])
        else:
            from pathlib import Path
            config_file = "{}/.aoa/config.yaml".format(Path.home())
            if os.path.isfile(config_file):
                self.__parse_yaml(config_file)

        self.__parse_env_variables()
        self.__parse_kwargs(**kwargs)

    def select_header_accept(self, accepts: List[str]):
        """
        converts list of header into a string

        Return:
            (str): request header
        """
        if not accepts:
            return
        accepts = [x.lower() for x in accepts]
        if 'application/json' in accepts:
            return 'application/json'
        else:
            return ', '.join(accepts)

    def get_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str]):
        """
        wrapper for get request

        Parameters:
           path (str): url
           header_params (dict): header parameters
           query_params (dict): query parameters

        Returns:
            dict for resources, str for errors, None for 404
        Raise:
            raises HTTPError in case of error status code other than 404
        """

        resp = requests.get(
            url=self.__strip_url(self.aoa_url) + path,
            auth=self.auth,
            headers=header_params,
            params=query_params
        )

        if resp.status_code == 404:
            return None

        resp.raise_for_status()

        try:
            return resp.json()
        except ValueError:
            return resp.text

    def post_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str], body: Dict[str, str]):
        """
        wrapper for post request

        Parameters:
           path (str): url
           header_params (dict): header parameters
           query_params (dict): query parameters
           body (dict): request body

        Returns:
            dict for resources, str for errors
        Raise:
            raises HTTPError in case of error status code
        """
        resp = requests.post(
            url=self.__strip_url(self.aoa_url) + path,
            auth=self.auth,
            headers=header_params,
            params=query_params,
            data=json.dumps(body)
        )

        resp.raise_for_status()

        try:
            return resp.json()
        except ValueError:
            return resp.text

    def put_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str], body: Dict[str, str]):
        """
        wrapper for put request

        Parameters:
           path (str): url
           header_params (dict): header parameters
           query_params (dict): query parameters
           body (dict): request body

        Returns:
            dict for resources, str for errors
        Raise:
            raises HTTPError in case of error status code
        """
        resp = requests.put(
            url=self.__strip_url(self.aoa_url) + path,
            auth=self.auth,
            headers=header_params,
            params=query_params,
            data=json.dumps(body)
        )

        resp.raise_for_status()
        try:
            return resp.json()
        except ValueError:
            return resp.text

    def __strip_url(self, url):
        return url.rstrip('/')


class AoaAuth(AuthBase):
    def __init__(self, aoa_credentials):
        self.aoa_credentials = aoa_credentials

    def __call__(self, r):
        r.headers['Authorization'] = "Basic {}".format(self.aoa_credentials)
        return r



import pytest
import time
from aoa.api.trained_model_api import TrainedModelApi
from aoa.api.job_api import JobApi
from aoa.api.job_event_api import JobEventApi
from aoa.api.trained_model_event_api import TrainedModelEventApi

max_retries = 10


def test_model_lifecycle(setup):
    trained_model_api = TrainedModelApi(pytest.aoa_client)
    job_api = JobApi(pytest.aoa_client)
    job_event_api = JobEventApi(pytest.aoa_client)
    trained_model_event_api = TrainedModelEventApi(pytest.aoa_client)
    event_status = []

    # Train
    training_request = dict()
    model_id = 'bf6a52b2-b595-4358-ac4f-24fb41a85c45'
    training_request['modelId'] = model_id
    training_request['datasetId'] = '11e1df4b-b630-47a1-ab80-7ad5385fcd8c'

    job_rtn = trained_model_api.train(training_request)
    job_id = job_rtn['id']
    job_rtn_by_id = job_api.find_by_id(job_id=job_id)

    assert (job_rtn_by_id['metadata']['trainingRequest'] == training_request), "validate training request...failed"

    retries = 0
    while True:
        job_event_rtn_by_id = job_event_api.find_by_job_id(job_id=job_id)
        event_status.clear()
        for event in job_event_rtn_by_id['_embedded']['jobEvents']:
            event_status.append(event['status'])
        if 'COMPLETED' in event_status:
            break
        elif retries > max_retries:
            assert ()
        else:
            time.sleep(5)
            retries += 1

    trained_model_id = job_rtn_by_id['metadata']['trainedModel']['id']
    trained_model_by_id = trained_model_api.find_by_id(trained_model_id=trained_model_id,
                                                       projection="expandTrainedModel")
    assert (trained_model_by_id['modelId'] == training_request['modelId'] and
            trained_model_by_id['dataset']['id'] == training_request['datasetId']), "train model...failed"

    # Evaluate
    evaluation_request = dict()
    evaluation_request['datasetId'] = '11e1df4b-b630-47a1-ab80-7ad5385fcd8c'

    job_eval_rtn = trained_model_api.evaluate(trained_model_id, evaluation_request)
    job_eval_id = job_eval_rtn['id']
    job_eval_rtn_by_id = job_api.find_by_id(job_id=job_eval_id)

    assert (job_eval_rtn_by_id['metadata']['evaluationRequest'] == evaluation_request), \
        "validate evaluate request...failed"

    retries = 0
    while True:
        job_eval_event_rtn_by_id = job_event_api.find_by_job_id(job_id=job_eval_id)
        event_status = []
        for event in job_eval_event_rtn_by_id['_embedded']['jobEvents']:
            event_status.append(event['status'])
        if 'COMPLETED' in event_status:
            break
        elif retries > max_retries:
            assert ()
        else:
            time.sleep(5)
            retries += 1

    trained_model_eval_by_id = trained_model_api.find_by_id(trained_model_id=trained_model_id,
                                                            projection="expandTrainedModel")
    assert (trained_model_eval_by_id['dataset']['id'] == evaluation_request['datasetId']), "evaluate model...failed"

    # Deploy
    transition_request = dict()
    transition_request['status'] = 'DEPLOYED'

    trained_model_api.transition(trained_model_id, transition_request)
    retries = 0
    while True:
        trained_event_by_id_rtn = trained_model_event_api.find_by_trained_model_id(trained_model_id=trained_model_id)
        event_status.clear()
        for event in trained_event_by_id_rtn['_embedded']['trainedModelEvents']:
            event_status.append(event['status'])
        if 'DEPLOYED' in event_status:
            break
        elif retries > max_retries:
            assert ()
        else:
            time.sleep(5)
            retries += 1

    # Retire
    transition_request = dict()
    transition_request['status'] = 'RETIRED'

    trained_model_api.transition(trained_model_id, transition_request)
    retries = 0
    while True:
        trained_event_by_id_rtn = trained_model_event_api.find_by_trained_model_id(trained_model_id=trained_model_id)
        event_status.clear()
        for event in trained_event_by_id_rtn['_embedded']['trainedModelEvents']:
            event_status.append(event['status'])
        if 'RETIRED' in event_status:
            break
        elif retries > max_retries:
            assert ()
        else:
            time.sleep(5)
            retries += 1

    assert (['TRAINED', 'EVALUATED', 'DEPLOYED', 'RETIRED'] == event_status), "deploy, retire model...failed"

    # Find by model id and status
    invalid_status_response = trained_model_api.find_by_model_id_and_status(model_id=model_id, status="DEPLOYED")
    assert (0 == invalid_status_response['page']['totalElements']), \
         "find by an invalid status should return nothing...passed"

    trained_model_by_model_id_and_status = trained_model_api.find_by_model_id_and_status(model_id=model_id,
                                                                                          status="RETIRED")
    assert (1 == trained_model_by_model_id_and_status['page']['totalElements']), "find by model id and status...failed"



import pandas as pd
from sklearn.neighbors import KNeighborsClassifier
from sklearn.externals import joblib
from sklearn import metrics
from sklearn.model_selection import train_test_split

# load data & engineer
df = pd.read_csv('https://datahub.io/machine-learning/iris/r/iris.csv')
# split dataset
train_df, predict_df = train_test_split(df, test_size = 0.5) 
features = 'sepallength,sepalwidth,petallength,petalwidth'.split(',')
X_train = train_df.loc[:, features]
y_train = train_df['class']

print("Starting training...")
# fit model to training data
knn = KNeighborsClassifier(n_neighbors=3)
knn.fit(X_train,y_train)
print("Finished training")

# evaluate model against test dataset
X_predict = predict_df.loc[:, features]
y_test = predict_df['class']
y_predict = knn.predict(X_predict)
print("model accuracy is ", metrics.accuracy_score(y_test, y_predict))

# save model
joblib.dump(knn, 'iris_knn.joblib')
print("Saved trained model")


from __future__ import absolute_import

from aoa.api.base_api import BaseApi


class TrainedModelEventApi(BaseApi):

    path = "/api/trainedModelEvents/"

    def find_all(self, projection: str = None, page: int = None, size: int = None, sort: str = None):
        """
        returns all trained model events

        Parameters:
           projection (str): projection type
           page (int): page number
           size (int): number of records in a page
           sort (str): column name and sorting order
           e.g. name?asc: sort name in ascending order, name?desc: sort name in descending order

        Returns:
            (dict): all trained model events
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id, 
            self.aoa_client.select_header_accept([
                'application/json', 
                'application/hal+json', 
                'text/uri-list', 
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection', 'page', 'sort', 'size', 'sort']
        query_vals = [projection, page, sort, size, sort]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path,
            header_params,
            query_params)

    def find_by_id(self, event_id: str, projection: str = None):
        """
        returns a trained model event

        Parameters:
           event_id (str): job id(uuid) to find
           projection (str): projection type

        Returns:
            (dict): trained model event
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + event_id,
            header_params,
            query_params)

    def find_by_status(self, status: str, projection: str = None, page: int = None, size: int = None,
                       sort: str = None):
        """
        returns trained model events by status

        Parameters:
           status (str): status of trained model event
            Available values : TRAINED, EVALUATED, APPROVED, REJECTED, CHAMPION, DEPLOYED, RETIRED
           projection (str): projection type
           page (int): page number
           size (int): number of records in a page
           sort (str): column name and sorting order
           e.g. name?asc: sort name in ascending order, name?desc: sort name in descending order

        Returns:
            (dict): trained model events
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['status', 'projection', 'page', 'sort', 'size', 'sort']
        query_vals = [status, projection, page, sort, size, sort]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + 'search/findByStatus',
            header_params,
            query_params)

    def find_by_trained_model_id(self, trained_model_id: str, projection: str = None,
                                 page: int = None, size: int = None, sort: str = None):
        """
        returns trained model events by trained model id

        Parameters:
           trained_model_id (str): trained model id(uuid)
           projection (str): projection type
           page (int): page number
           size (int): number of records in a page
           sort (str): column name and sorting order
           e.g. name?asc: sort name in ascending order, name?desc: sort name in descending order

        Returns:
            (dict): trained model events
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['trainedModelId', 'projection', 'page', 'sort', 'size', 'sort']
        query_vals = [trained_model_id, projection, page, sort, size, sort]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + 'search/findByTrainedModelId',
            header_params,
            query_params)



from aoa.model_utility.base_model import BaseModel
from aoa.model_utility.model_utility import ModelUtility
import json
import os
import subprocess
import sys
import logging


class EvaluateModel(BaseModel):

    def __init__(self, model_utility: ModelUtility):
        super().__init__(model_utility)
        self.logger = logging.getLogger(__name__)

    def evaluate_model_local(self, model_id: str = None, base_path: str = None, data_conf: dict = None):
        base_path = self.model_utility.model_catalog_path if base_path is None else os.path.join(base_path, '')
        model_path = base_path + 'model_definitions/'

        if not os.path.exists(model_path):
            raise ValueError("model directory {0} does not exist".format(model_path))

        if model_id is None:
            if "id" in self.model_utility.model:
                model_id = self.model_utility.model["id"]
            else:
                BaseModel.get_model_id(model_path)

        model_artefacts_path = ".artefacts/{}".format(model_id)

        try:
            os.symlink(model_artefacts_path, "./models", target_is_directory=True)

            model_dir = model_path + model_id

            cli_model_kargs = {
                "model_id": model_id,
                "model_version": "cli",
                "model_table": "AOA_MODELS_cli"
            }

            with open(model_dir + "/model.json", 'r') as f:
                model_definition = json.load(f)

            with open(model_dir + "/config.json", 'r') as f:
                model_conf = json.load(f)

            self.logger.info("Loading and executing model code")

            engine = self._BaseModel__get_engine(model_definition)
            if engine == "python" or engine == "pyspark":
                if engine == "pyspark":
                    self._BaseModel__configure_pyspark(model_definition, model_artefacts_path)

                sys.path.append(model_dir)
                import model_modules
                model_modules.scoring.evaluate(data_conf, model_conf, **cli_model_kargs)

            elif engine == "sql":
                self.__evaluate_sql(model_dir, data_conf, model_conf, **cli_model_kargs)

            elif engine == "R":
                cmd = self.model_utility.dir_path + "/run_model.R {} {} {} {}".format(model_id, "evaluate", dataset_path, base_path)
                subprocess.check_call(cmd, shell=True)

            else:
                raise Exception("Unsupported language: {}".format(model_definition["language"]))

            self.logger.info("Artefacts can be found in: {}".format(model_artefacts_path))
            self.logger.info("Evaluation metrics can be found in: {}/evaluation.json".format(model_artefacts_path))
            self.__cleanup()
        except:
            self.__cleanup()
            raise

    def __cleanup(self):
        os.remove("./models")

    def __evaluate_sql(self, model_dir, data_conf, model_conf, **kwargs):
        from teradataml import create_context
        from teradataml.dataframe.dataframe import DataFrame
        from teradataml.context.context import get_connection

        self.logger.info("Starting evaluation...")

        create_context(host=data_conf["hostname"],
                       username=os.environ["TD_USERNAME"],
                       password=os.environ["TD_PASSWORD"],
                       logmech=os.getenv("TD_LOGMECH", "TDNEGO"))

        sql_file = model_dir + "/model_modules/scoring.sql"
        jinja_ctx = {
            "data_conf": data_conf,
            "model_conf": model_conf,
            "model_table": kwargs.get("model_table"),
            "model_version": kwargs.get("model_version"),
            "model_id": kwargs.get("model_id")
        }

        self._BaseModel__execute_sql_script(get_connection(), sql_file, jinja_ctx)

        self.logger.info("Finished evaluation")

        stats = DataFrame(data_conf["metrics_table"]).to_pandas()
        metrics = dict(zip(stats.key, stats.value))

        with open("models/evaluation.json", "w+") as f:
            json.dump(metrics, f)

        self.logger.info("Saved metrics")



from __future__ import absolute_import

from aoa.api.base_api import BaseApi


class JobApi(BaseApi):

    path = "/api/jobs/"

    def find_all(self, projection: str = None, page: int = None, size: int = None, sort: str = None):
        """
        returns all jobs

        Parameters:
           projection (str): projection type
           page (int): page number
           size (int): number of records in a page
           sort (str): column name and sorting order
           e.g. name?asc: sort name in ascending order, name?desc: sort name in descending order

        Returns:
            (dict): all jobs
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection', 'sort', 'page', 'size', 'sort']
        query_vals = [projection, sort, page, size, sort]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path,
            header_params,
            query_params)

    def find_by_id(self, job_id: str, projection: str = None):
        """
        returns a job

        Parameters:
           job_id (str): job id(uuid) to find
           projection (str): projection type

        Returns:
            (dict): job
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + job_id,
            header_params,
            query_params)

    def find_job_events(self, job_id: str, projection: str = None):
        """
        returns events of a job

        Parameters:
           job_id (str): job id(uuid) to find events
           projection (str): projection type

        Returns:
            (dict): job events
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + job_id + '/events',
            header_params,
            query_params)

    def find_by_job_event_id(self, job_id: str, job_event_id: str, projection: str = None):
        """
        returns job event

        Parameters:
           job_id (str): job id(uuid)
           job_event_id (str): job event id(uuid)
           projection (str): projection type

        Returns:
            (dict): job event
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + job_id + '/events/' + job_event_id,
            header_params,
            query_params)

    def find_job_between_intervals(self, start_time: str, end_time: str,
                                   projection: str = None):
        """
        returns job events within the timestamp range

        Parameters:
           start_time (str): start time e.g: 2020-03-18T09:05:03.569Z
           end_time (str): end time e.g: 2020-03-24T09:05:03.569Z
           projection (str): projection type

        Returns:
            (dict): job events
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['startAt', 'endAt', 'projection']
        query_vals = [start_time, end_time, projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + "search/findByCreatedAtBetween",
            header_params,
            query_params)

    def find_by_status(self, status: str, projection: str = None, page: int = None, size: int = None,
                       sort: str = None):
        """
        returns job by status

        Parameters:
           status (str): job status
           projection (str): projection type
           page (int): page number
           size (int): number of records in a page
           sort (str): column name and sorting order
           e.g. name?asc: sort name in ascending order, name?desc: sort name in descending order

        Returns:
            (dict): job events
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['status', 'projection', 'page', 'sort', 'size', 'sort']
        query_vals = [status, projection, page, sort, size, sort]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + 'search/findByStatusIn',
            header_params,
            query_params)





def evaluate(data_conf, model_conf, **kwargs):
    """Python evaluate method called by AOA framework

    Parameters:
    data_conf (dict): The dataset metadata
    model_conf (dict): The model configuration to use

    Returns:
    None:No return

    """

    # dump results as json file evaluation.json to models/ folder
    print("Evaluation complete...")



import pytest
import uuid
from aoa.api.dataset_api import DatasetApi


def _skip_generated_fields(dataset):
    keys_to_remove = ('id', 'ownerId', 'createdAt', '_links', 'projectId')
    for k in keys_to_remove:
        dataset.pop(k, None)
    return dataset


def test_dataset(setup):
    dataset_api = DatasetApi(pytest.aoa_client)
    before_count = len(dataset_api.find_all()['_embedded']['datasets'])

    dataset_definition = {
        "name": "new dataset",
        "description": "adding sample dataset",
        "metadata": {
            "url": "http://nrvis.com/data/mldata/pima-indians-diabetes.csv",
            "test_split": "0.2"
        }
    }

    dataset_rtn = dataset_api.save(dataset_definition)

    after_count = len(dataset_api.find_all()['_embedded']['datasets'])
    assert ( before_count + 1 == after_count), "save dataset...failed"

    dataset_rtn_by_id = dataset_api.find_by_id(dataset_id=dataset_rtn['id'])
    assert (_skip_generated_fields(dataset_rtn_by_id) == dataset_definition), "validate new dataset...failed"

    dataset_by_name = dataset_api.find_by_name(dataset_name=dataset_definition["name"])
    assert (_skip_generated_fields(dataset_by_name) == dataset_definition), "find dataset by name...failed"

    dataset_rtn_by_id = dataset_api.find_by_id(dataset_id=str(uuid.uuid4()))
    assert (dataset_rtn_by_id is None), "validate missing resouce... failed"





import pandas as pd
from sklearn import metrics
from sklearn.externals import joblib
import json

def evaluate(data_conf, model_conf, **kwargs):
    """Python evaluate method called by AOA framework

    Parameters:
    data_conf (dict): The dataset metadata
    model_conf (dict): The model configuration to use

    Returns:
    None:No return

    """

    predict_df = pd.read_csv(data_conf['location'])
    features = 'sepallength,sepalwidth,petallength,petalwidth'.split(',')
    X_predict = predict_df.loc[:, features]
    y_test = predict_df['class']
    knn = joblib.load('models/iris_knn.joblib')

    y_predict = knn.predict(X_predict)
    scores = {}
    scores['accuracy'] = metrics.accuracy_score(y_test, y_predict)
    print("model accuracy is ", scores['accuracy'])

    # dump results as json file evaluation.json to models/ folder
    with open("models/evaluation.json", "w+") as f:
        json.dump(scores, f)
    print("Evaluation complete...")


import pytest
from aoa.api.project_api import ProjectApi


def test_project(setup):
    project_api = ProjectApi(pytest.aoa_client)
    before_count = len(project_api.find_all()['_embedded']['projects'])

    project = dict()
    project['name'] = 'new_project'
    project['gitRepositoryUrl'] = "https://github.com/ThinkBigAnalytics/TestAoaDemoModels.git"
    project['groupId'] = 'DEMO'
    project['description'] = 'new_project'
    project_rtn = project_api.save(project)
    project_id = project_rtn['id']

    after_count = len(project_api.find_all()['_embedded']['projects'])

    assert (before_count + 1 == after_count), "save project...failed"

    project_rtn_by_id = project_api.find_by_id(project_id=project_id)
    key_to_remove = ('id', 'createdAt', '_links')
    for k in key_to_remove:
        project_rtn_by_id.pop(k, None)

    assert (project_rtn_by_id == project), "validate new project...failed"

    project_name = 'project_updated'
    pytest.aoa_client.set_project_id(project_id)
    project_upd = dict()
    project_upd['name'] = project_name
    project_upd['gitRepositoryUrl'] = "https://github.com/ThinkBigAnalytics/AoaDemoModelsUpdated.git"
    project_upd['groupId'] = 'DEMO'
    project_upd['description'] = project_name
    project_api.update(project_upd)

    project_rtn_upd_by_id = project_api.find_by_id(project_id=project_id)
    key_to_remove = ('id', 'createdAt', '_links')
    for k in key_to_remove:
        project_rtn_upd_by_id.pop(k, None)

    assert (project_upd == project_rtn_upd_by_id), 'validate updated project...failed'




from __future__ import absolute_import
from typing import Dict

from aoa.api.base_api import BaseApi


class ProjectApi(BaseApi):

    path = "/api/projects/"

    def find_all(self, projection: str = None, page: int = None, size: int = None, sort: str = None):
        """
        returns all projects

        Parameters:
           projection (str): projection type
           page (int): page number
           size (int): number of records in a page
           sort (str): column name and sorting order
           e.g. name?asc: sort name in ascending order, name?desc: sort name in descending order

        Returns:
            (dict): all projects
        """
        header_vars = ['Accept']
        header_vals = ['application/json']
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection', 'page', 'sort', 'size', 'sort']
        query_vals = [projection, page, sort, size, sort]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path,
            header_params,
            query_params)
    
    def find_by_id(self, project_id: str, projection: str = None):
        """
        returns a project

        Parameters:
           project_id (str): project id(uuid) to find
           projection (str): projection type

        Returns:
            (dict): project
        """
        header_vars = ['Accept']
        header_vals = ['application/json']
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + project_id,
            header_params,
            query_params)
    
    def save(self, project: Dict[str, str]):
        """
        create a project

        Parameters:
           project (dict): project to create

        Returns:
            (dict): project
        """
        header_vars = ['Accept']
        header_vals = ['application/json']
        header_params = self.generate_params(header_vars, header_vals)

        query_params = {}

        self.required_params(['description', 'gitRepositoryUrl', 'groupId', 'name'], project)

        return self.aoa_client.post_request(
            self.path,
            header_params,
            query_params,
            project)

    def update(self, project: Dict[str, str]):
        """
        update a project

        Parameters:
           project (dict): project to update

        Returns:
            (dict): project
        """
        header_vars = ['Accept']
        header_vals = ['application/json']
        header_params = self.generate_params(header_vars, header_vals)

        query_params = {}

        self.required_params(['description', 'gitRepositoryUrl', 'groupId', 'name'], project)

        return self.aoa_client.put_request(
            self.path + self.aoa_client.project_id,
            header_params,
            query_params,
            project)



from aoa.model_utility.model_utility import ModelUtility
import logging
import json
import os
from jinja2 import Template


class BaseModel(object):

    def __init__(self, model_utility: ModelUtility):
        self.model_utility = model_utility

    @staticmethod
    def get_model_id(model_path, rtn_val=False):
        catalog = {}
        index = 0
        model_ids = "Use one of the following models\n"

        for model in os.listdir(model_path):
            if os.path.exists(model_path + model + "/model.json"):
                with open(model_path + model + "/model.json", 'r') as f:
                    model_definition = json.load(f)
                    catalog[index] = model_definition
                    index += 1

        for key in catalog:
            model_ids += "{1}: {0}\n".format(catalog[key]["id"], catalog[key]["name"])

        if rtn_val:
            return catalog

        raise ValueError(model_ids)

    def __template_sql_script(self, filename, jinja_ctx):
        with open(filename) as f:
            template = Template(f.read())

        return template.render(jinja_ctx)

    def __execute_sql_script(self, conn, filename, jinja_ctx):
        script = self.__template_sql_script(filename, jinja_ctx)

        stms = script.split(';')

        for stm in stms:
            stm = stm.strip()
            if stm:
                logging.info("Executing statement: {}".format(stm))

                try:
                    conn.execute(stm)
                except Exception as e:
                    if stm.startswith("DROP"):
                        logging.warning("Ignoring DROP statement exception")
                    else:
                        raise e

    def __configure_pyspark(self, model_definition, model_artefacts_path):
        import findspark
        import os

        # read the spark submit resources for the model
        if not model_definition["resources"] or not model_definition["resources"]["training"]:
            raise Exception("pyspark models require a resources->training section in model.json")

        resources = model_definition["resources"]["training"]
        aoa_spark_conf = os.environ.get("AOA_SPARK_CONF", "--conf spark.aoa.modelPath={}".format(model_artefacts_path))

        os.environ["PYSPARK_SUBMIT_ARGS"] = "--master {} {} {}  pyspark-shell".format(
            resources["master"], resources["args"], aoa_spark_conf)

        # SPARK_HOME should be set
        spark_home = findspark.find()
        findspark.init(spark_home)

        self.logger.info("Using SPARK_HOME: {}".format(spark_home))
        self.logger.info("Using PYSPARK_SUBMIT_ARGS: {}".format(os.environ["PYSPARK_SUBMIT_ARGS"]))
        self.logger.info("Using AOA_SPARK_CONF: {}".format(os.environ.get("AOA_SPARK_CONF", "")))

    def __get_engine(self, model_definition):
        if "automation" in model_definition \
                and "trainingEngine" in model_definition["automation"]:
            return model_definition["automation"]["trainingEngine"]

        return model_definition["language"]


import pytest
from aoa.api.model_api import ModelApi


def test_model(setup):
    model_api = ModelApi(pytest.aoa_client)
    models_all_rtn = model_api.find_all()

    if len(models_all_rtn) > 0:
        model_rtn = models_all_rtn[0]

    model_by_id_rtn = model_api.find_by_id(model_id=model_rtn['id'])

    assert (model_rtn == model_by_id_rtn), "validate models...failed"
